function DDS(sinitial, its, fraction1, maxiter, obj_flag, Select_metric,Eps,MinPts, out_plot)   %updated cluster
% Dynamically Dimensioned Search algorithm was initially developed by Bryan
% Tolson. Version 1.0.2  Oct 2005 Coded as a MINIMZER but variable
% obj_flag converts a maximization objective to a minimization one without
% modifying the code below. In other words, specify obj_flag in DDS_inp.txt
% correctly and the code here does not need modification for MAX or MIN
% objectives. this algorithm records all sampled solutions but only
% searches based on the best solution. This algorithm was then improved by
% Dr. Masoud Asadzadeh (2014) for solving the multi-objective optimization
% case studies. The modifications and improvements by Dr. Masoud Asadzadeh
% are explained in the following.
% Modified by Dr. Masoud Asadzadeh for PA-DDS Feb 2014:
% The modifications indlude:
%   1- The original perturbation which was designed for real-valued DVs is
%   modified to a mixed perturbation that can handle both real and discrete
%   valued decision variables.
%   2- The archiving strategy is added. For now all the non-dominated
%   solutions are archived.
%   3- The selection metric calculation function is added (RND: random
%   selection, CRD: Crowding Distance, HVC: Hypervolume selection)
%   4- The selection mechanism is added (This selection can be based on CD,
%   random, or ...)
%   5- Plotting the Pareto front with up to 6 objective functions.
% The structure of multi-objective algorithm was changed by Shahram Sahraei
% (2018) to archive near-optimal along with optimal tradeoffs but with
% distinct characteristics in the variable space. The new diversity-based
% PA-DDS works based on the diversity in the variable space. In order to
% include the decision space diversity in the optimization, a density-based
% spatial clustering method is used. To apply the spatial clustering to the
% solutions in the decision space, it is first required to specify the
% minimum adjacency density parameters for initial cluster formation
% including MinPts and Eps. If the minimum adjecency density is met, the
% solution that covers at least MinPts solution, including itself, in a 
% domain of Eps is called core solution and the other solutions that are
% density-reachable from the core solution but themselves cannot satisfy
% the minimum adjacency density are called border points in that cluster.
% If a solution cannot satisfy the minimum adjacency density nor is
% density-reachable from a core solution, is called a noise solutions. The
% core and border solutions are given a unique label different from other
% groups upon forming a new cluster. A local dominance check is then
% applied to the solutions with the same cluster label and no comparison is
% made between solutions with different cluster labels. The dominated
% solutions are discarded from the cluster and also from the archive and
% the non-dominated solutions are retained with their accumulative
% adjacency density they have achieved so far from the beginning of the
% optimization in order to allow for a possible cluster expansion and
% neighborhood clusters to merge in the remaining iterations of the
% optimization process. Solutions with no cluster ID, i.e. noises, are also
% preserved in the archive set for the possible formation of new clusters
% in subsequent iterations.
% There are variables with structure format that can be seen throughout the
% diversity-based PA-DDS algorithm. The content of these variables may have 
% *.Value, *.ClID, and *.Type, which respectively contain the position of
% solutions in the decision space as well as the objective space, the 
% corresponding cluster label, and the cumulative adjecency density for 
% each solution. each row in each substructure is representative of a 
% specific solution. *.Value substructure is a Matrix and *.ClID and *.Type
% are vectors.

global num_dec;
global S_min; % row vector of dec variable minimums - get from main program
global S_max; % row vector of dec variable maximums - get from main program

global num_objs; %added by masoud 09/05/08
global Discrete_flag;
global PF_set;
global objfunc_name; % function name that returns the objective function value
global Modeldir;

main_dir = pwd;
%% Variables Initialization
Jtest = zeros(1,num_objs);
stest = zeros(1,num_dec);
S_range = S_max - S_min;
NewClID=1; %Added by Shahram Sahraei for cluster-based PA-DDS (Dec. 1, 2017)
%% Initialization

if its > 1   % "its" is the number of function evaluations to initialize the DDS algorithm solution
	ileft = maxiter - its; % use this to reduce number of fevals in DDS loop
    if ileft <= 0
        error('#Initialization samples >= Max # of function evaluations.');
    end
    Initial_Sol = LHSU(S_min,S_max,its,Discrete_flag);
    for i = 1 : its
        stest=Initial_Sol(i,:);
        cd(Modeldir);
        Jtest = feval(objfunc_name,stest);
        Jtest = obj_flag .* Jtest;
        cd(main_dir);
        if i == 1
            %NewSol is the currently generated new solution
            NewSol.Value=[Jtest, stest]; %Added by Shahram Sahraei for cluster-based PA-DDS (Dec. 1, 2017)
            NewSol.ClID=0; %Added by Shahram Sahraei for cluster-based PA-DDS (Dec. 1, 2017)
            NewSol.Type=1; %Added by Shahram Sahraei for cluster-based PA-DDS (Dec. 1, 2017)
            PF_set = NewSol; %The first solution is obviously non-dominated
            temp = obj_flag .* Jtest;
            temp = double(temp);
            save nd_pts.mat temp -ascii
            temp = stest;
            temp = double(temp);
            save nd_sol.mat temp -ascii
            temp=0;  %Added by Shahram Sahraei for cluster-based PA-DDS (Dec. 1, 2017)
            save nd_ClID.mat temp -ascii   %Added by Shahram Sahraei for cluster-based PA-DDS (Dec. 1, 2017)
            temp=1;  %Added by Shahram Sahraei for cluster-based PA-DDS (Dec. 1, 2017)
            save nd_Type.mat temp -ascii   %Added by Shahram Sahraei for cluster-based PA-DDS (Dec. 1, 2017)
            
        else %check whether the solution is dominated
            NewSol.Value=[Jtest, stest]; %Added by Shahram Sahraei for cluster-based PA-DDS (Dec. 1, 2017)
            NewSol.ClID=0; %Added by Shahram Sahraei for cluster-based PA-DDS (Dec. 1, 2017)
            NewSol.Type=0; %Added by Shahram Sahraei for cluster-based PA-DDS (Dec. 1, 2017)
            SetOfSols=struct('Value',PF_set.Value(:,num_objs+1:end),'ClID',PF_set.ClID','Type',PF_set.Type'); %Updated by Shahram Sahraei for cluster-based PA-DDS (Dec. 1, 2017)
            NewOne=struct('Value',NewSol.Value(num_objs+1:end),'ClID',NewSol.ClID,'Type',NewSol.Type); %Updated by Shahram Sahraei for cluster-based PA-DDS (Dec. 1, 2017)
            % normalize the cluster non-dominated solutions in SetOfSols and the new Solution in NewOne.
            SetOfSols.Value=(SetOfSols.Value-repmat(S_min,size(SetOfSols.Value,1),1))./repmat(S_max-S_min,size(SetOfSols.Value,1),1); %Updated by Shahram Sahraei for cluster-based PA-DDS (Dec. 1, 2017)
            NewOne.Value=(NewOne.Value-S_min)./(S_max-S_min); %Added by Shahram Sahraei for cluster-based PA-DDS (Dec. 1, 2017)
            % Check whether new cluster formation or expansion of the
            % previously formed cluster occurs. The function of
            % ExpandCluster updates clusters.
            [Logic, SetOfSols, ChangedSols,Last_label]=ExpandCluster(SetOfSols,NewOne,NewClID,Eps,MinPts); %Added by Shahram Sahraei for cluster-based PA-DDS (Dec. 1, 2017)
            % If two or more clusters merge, the cluster IDs are relabeled
            % from 1 to the number of the existing clusters contiguously to
            % have no discontinuity in the cluster labels.
%%%%%%            [SetOfSols.ClID, Last_label, NewOne.ClID]=relabel1(SetOfSols.ClID);  %Added by Shahram Sahraei for cluster-based PA-DDS (Dec. 1, 2017)
            % Convert back the values of the variables of the archived 
            % solutions to their unnormalized values.
            SetOfSols.Value=(SetOfSols.Value.*repmat(S_max-S_min,size(SetOfSols.Value,1),1)+repmat(S_min,size(SetOfSols.Value,1),1)); %Added by Shahram Sahraei for cluster-based PA-DDS (Dec. 1, 2017)
            % ChangedSols are the solutions that a change in their cluster
            % labels is made due to the new solution.
            ChangedSols.Value=(ChangedSols.Value.*repmat(S_max-S_min,size(ChangedSols.Value,1),1)+repmat(S_min,size(ChangedSols.Value,1),1)); %Added by Shahram Sahraei for cluster-based PA-DDS (Dec. 1, 2017)
            NewSol.ClID=SetOfSols.ClID(end);   %Added by Shahram Sahraei for cluster-based PA-DDS (Dec. 1, 2017)
            NewSol.Type=SetOfSols.Type(end);   %Added by Shahram Sahraei for cluster-based PA-DDS (Dec. 1, 2017)
            % put the new solution with its cluster label and adjacency
            % density inside the Pareto set.
            PF_set.Value=[PF_set.Value;NewSol.Value]; %Added by Shahram Sahraei for cluster-based PA-DDS (Dec. 1, 2017)
            PF_set.ClID=SetOfSols.ClID'; %Added by Shahram Sahraei for cluster-based PA-DDS (Dec. 1, 2017)
            PF_set.Type=SetOfSols.Type'; %Added by Shahram Sahraei for cluster-based PA-DDS (Dec. 1, 2017)
            % find all the solutions inside PF_set that are same as
            % ChangedSol and insert them into the ChangedSols structure.
            % The reason is to prevent duplication since sometimes happens 
            % that the new solution becomes identical with an archived one.
            ind=ismembertol(PF_set.Value(:,num_objs+1:end),ChangedSols.Value,0.00000001,'ByRows',true); %Added by Shahram Sahraei for cluster-based PA-DDS (Dec. 1, 2017)
            ind2=find(ind==1); %Added by Shahram Sahraei for cluster-based PA-DDS (Dec. 1, 2017)
            ind3=PF_set.ClID(ind2)~=unique(ChangedSols.ClID);
            ind2(ind3)=[];
            a=struct('Value',PF_set.Value(ind2,:),'ClID',PF_set.ClID(ind2),'Type',PF_set.Type(ind2)); %Added by Shahram Sahraei for cluster-based PA-DDS (Dec. 1, 2017)
            [ChangedSols.Value,ia,~]=unique(a.Value,'rows','stable');
            ChangedSols.ClID=a.ClID(ia);
            ChangedSols.Type=a.Type(ia);
            % If a new cluster by new solution is formed, add one to the
            % biggest cluster label.
            if isequal(Logic,1)  %Added by Shahram Sahraei for cluster-based PA-DDS (Dec. 1, 2017)
                NewClID=Last_label+1;  %Added by Shahram Sahraei for cluster-based PA-DDS (Dec. 1, 2017)
            end
            % if more than one solution have the changed cluster ID, first
            % compare them together and then apply cluster-dominance to all
            % the solutions in PF_set with the same label as ChangedSols.
            if size(a.Value,1)>1 %Added by Shahram Sahraei for cluster-based PA-DDS (Dec. 1, 2017)
                temp1=ChangedSols;
                ChangedSols = ND_check_cluster_prime(ChangedSols,ChangedSols,num_objs);%Added by Shahram Sahraei for cluster-based PA-DDS (Dec. 1, 2017)   
                % Find the cluster-dominated solutions in PF_set and delete them.
                ind=ismembertol(temp1.Value(:,1:num_objs),ChangedSols.Value(:,1:num_objs),0.00000001,'Byrows',true);
                ind2=find(ind==1);
                temp1.Value(ind2,:)=[];temp1.ClID(ind2)=[];temp1.Type(ind2)=[];
                ind=ismembertol(PF_set.Value(:,1:num_objs),temp1.Value(:,1:num_objs),0.00000001,'Byrows',true);
                ind2=find(ind==1);
                ind3=PF_set.ClID(ind2)~=unique(temp1.ClID);
                ind2(ind3)=[];
                PF_set.Value(ind2,:)=[];PF_set.ClID(ind2)=[];PF_set.Type(ind2)=[];
            end
            % Sometimes, the new solution becomes a member of cluster and 
            % no other change is made to other solutions nearby the new
            % solution. In this situation, the new cluster member should be
            % compared with its groupmates in terms of cluster-dominance
            % relations.There occurs situations that two or more clusters
            % merge due to the new solution. In this situation, all the
            % solutions inside those clusters that are now merged should be
            % compared mutually in terms of dominance on more time. When a
            % merge happens, the function ExpandCluster updates the cluster
            % labels in PF_set but all the solutions with the changed
            % cluster labels may not be reported in ChangedSols. That is why
            % the following dominance check is again required to take care
            % of it.
            if ~isequal(unique(ChangedSols.ClID),0) %Added by Shahram Sahraei for cluster-based PA-DDS (Dec. 1, 2017)
                PF_set = ND_check_cluster_prime(PF_set, ChangedSols, num_objs);%Added by Shahram Sahraei for cluster-based PA-DDS (Dec. 1, 2017)
            end
            %save the cluster-non-dominated points, their labels and their
            % cumulative adjacency densities.
            temp = repmat(obj_flag, size(PF_set.Value,1), 1) .* PF_set.Value(:,1:num_objs);  %Updated by Shahram Sahraei for cluster-based PA-DDS (Dec. 1, 2017)
            temp = double(temp);
            save nd_pts.mat temp -ascii  
            temp = PF_set.Value(:,num_objs+1:num_objs+num_dec);  %Updated by Shahram Sahraei for cluster-based PA-DDS (Dec. 1, 2017)
            temp = double(temp);
            save nd_sol.mat temp -ascii 
            temp = PF_set.ClID;  %Added by Shahram Sahraei for cluster-based PA-DDS (Dec. 1, 2017)
            temp = double(temp); %Added by Shahram Sahraei for cluster-based PA-DDS (Dec. 1, 2017)
            save nd_ClID.mat temp -ascii   %Added by Shahram Sahraei for cluster-based PA-DDS (Dec. 1, 2017)
            temp = PF_set.Type;  %Added by Shahram Sahraei for cluster-based PA-DDS (Dec. 1, 2017)
            temp = double(temp); %Added by Shahram Sahraei for cluster-based PA-DDS (Dec. 1, 2017)
            save nd_Type.mat temp -ascii  %Added by Shahram Sahraei for cluster-based PA-DDS (Dec. 1, 2017)

%% Plotting Initial solutions. Turn off for long runs
            if out_plot == 1
                Plot_tradeoff(obj_flag, PF_set.Value, [], [], objfunc_name, i, maxiter);  %Updated Cluster
                pause(0.001);
            end
        end        
    end
else  % --> its=1 --> using a set of user supplied initial solutions.
    % All the above explanations applies to the rest of the code.
    for i = 1 : size(sinitial,1)
        if size(sinitial, 2) == (num_objs + num_dec)
            Jtest = obj_flag .* sinitial(i, 1 : num_objs);
            stest = sinitial(i, num_objs + 1 : num_objs + num_dec);
            its = 0;
        else
            stest = sinitial(i,:);  % get from the inputs
            cd(Modeldir);
            Jtest = feval(objfunc_name,stest); %Calculate obj func value
            Jtest = obj_flag .* Jtest;
            cd(main_dir);
            its = size(sinitial, 1);
        end
        if i == 1
            NewSol.Value=[Jtest, stest]; %Added by Shahram Sahraei for cluster-based PA-DDS (Dec. 1, 2017)
            NewSol.ClID=0; %Added by Shahram Sahraei for cluster-based PA-DDS (Dec. 1, 2017)
            NewSol.Type=1; %Added by Shahram Sahraei for cluster-based PA-DDS (Dec. 1, 2017)
            PF_set = NewSol; %Added by Shahram Sahraei for cluster-based PA-DDS (Dec. 1, 2017)
            temp = obj_flag .* Jtest;
            temp = double(temp);
            save nd_pts.mat temp -ascii
            temp = stest;
            temp = double(temp);
            save nd_sol.mat temp -ascii
            temp=0;  %Added by Shahram Sahraei for cluster-based PA-DDS (Dec. 1, 2017)
            save nd_ClID.mat temp -ascii  %Added by Shahram Sahraei for cluster-based PA-DDS (Dec. 1, 2017)
            temp=1;  %Added by Shahram Sahraei for cluster-based PA-DDS (Dec. 1, 2017)
            save nd_Type.mat temp -ascii  %Added by Shahram Sahraei for cluster-based PA-DDS (Dec. 1, 2017)
        else
            NewSol.Value=[Jtest, stest]; %Added by Shahram Sahraei for cluster-based PA-DDS (Dec. 1, 2017)
            NewSol.ClID=0; %Added by Shahram Sahraei for cluster-based PA-DDS (Dec. 1, 2017)
            NewSol.Type=0; %Added by Shahram Sahraei for cluster-based PA-DDS (Dec. 1, 2017)
            SetOfSols=struct('Value',PF_set.Value(:,num_objs+1:end),'ClID',PF_set.ClID','Type',PF_set.Type'); %Added by Shahram Sahraei for cluster-based PA-DDS (Dec. 1, 2017)
            NewOne=struct('Value',NewSol.Value(num_objs+1:end),'ClID',NewSol.ClID,'Type',NewSol.Type); %Added by Shahram Sahraei for cluster-based PA-DDS (Dec. 1, 2017)
            SetOfSols.Value=(SetOfSols.Value-repmat(S_min,size(SetOfSols.Value,1),1))./repmat(S_max-S_min,size(SetOfSols.Value,1),1); %Added by Shahram Sahraei for cluster-based PA-DDS (Dec. 1, 2017)
            NewOne.Value=(NewOne.Value-S_min)./(S_max-S_min); %Added by Shahram Sahraei for cluster-based PA-DDS (Dec. 1, 2017)
            [Logic, SetOfSols,ChangedSols,Last_label]=ExpandCluster(SetOfSols,NewOne,NewClID,Eps,MinPts); %Added by Shahram Sahraei for cluster-based PA-DDS (Dec. 1, 2017)
%%%%%%            [SetOfSols.ClID, Last_label, NewOne.ClID]=relabel1(SetOfSols.ClID);  %Added by Shahram Sahraei for cluster-based PA-DDS (Dec. 1, 2017)
            SetOfSols.Value=(SetOfSols.Value.*repmat(S_max-S_min,size(SetOfSols.Value,1),1)+repmat(S_min,size(SetOfSols.Value,1),1)); %Added by Shahram Sahraei for cluster-based PA-DDS (Dec. 1, 2017)
            ChangedSols.Value=(ChangedSols.Value.*repmat(S_max-S_min,size(ChangedSols.Value,1),1)+repmat(S_min,size(ChangedSols.Value,1),1)); %Added by Shahram Sahraei for cluster-based PA-DDS (Dec. 1, 2017)
            NewSol.ClID=SetOfSols.ClID(end);   %Added by Shahram Sahraei for cluster-based PA-DDS (Dec. 1, 2017)
            NewSol.Type=SetOfSols.Type(end);   %Added by Shahram Sahraei for cluster-based PA-DDS (Dec. 1, 2017)
            PF_set.Value=[PF_set.Value;NewSol.Value];
            PF_set.ClID=SetOfSols.ClID'; %Added by Shahram Sahraei for cluster-based PA-DDS (Dec. 1, 2017)
            PF_set.Type=SetOfSols.Type'; %Added by Shahram Sahraei for cluster-based PA-DDS (Dec. 1, 2017)
            ind=ismembertol(PF_set.Value(:,num_objs+1:end),ChangedSols.Value,0.00000001,'ByRows',true); %Added by Shahram Sahraei for cluster-based PA-DDS (Dec. 1, 2017)
            ind2=find(ind==1); %Added by Shahram Sahraei for cluster-based PA-DDS (Dec. 1, 2017)
            ind3=PF_set.ClID(ind2)~=unique(ChangedSols.ClID);
            ind2(ind3)=[];
            a=struct('Value',PF_set.Value(ind2,:),'ClID',PF_set.ClID(ind2),'Type',PF_set.Type(ind2)); %Added by Shahram Sahraei for cluster-based PA-DDS (Dec. 1, 2017)
            [ChangedSols.Value,ia,~]=unique(a.Value,'rows','stable');
            ChangedSols.ClID=a.ClID(ia);
            ChangedSols.Type=a.Type(ia);
            if isequal(Logic,1)  %Added by Shahram Sahraei for cluster-based PA-DDS (Dec. 1, 2017)
                NewClID=Last_label+1;  %Added by Shahram Sahraei for cluster-based PA-DDS (Dec. 1, 2017)
            end 
            if size(a.Value,1)>1 %Added by Shahram Sahraei for cluster-based PA-DDS (Dec. 1, 2017)
                temp1=ChanedSols;
                ChangedSols = ND_check_cluster_prime(ChangedSols,ChangedSols, num_objs); %Added by Shahram Sahraei for cluster-based PA-DDS (Dec. 1, 2017)  
                ind=ismembertol(temp1.Value(:,1:num_objs),ChangedSols.Value(:,1:num_objs),0.00000001,'Byrows',true);
                ind2=find(ind==1);
                temp1.Value(ind2,:)=[];temp1.ClID(ind2)=[];temp1.Type(ind2)=[];
                ind=ismembertol(PF_set.Value(:,1:num_objs),temp1.Value(:,1:num_objs),0.00000001,'Byrows',true);
                ind2=find(ind==1);
                ind3=PF_set.ClID(ind2)~=unique(temp1.ClID);
                ind2(ind3)=[];
                PF_set.Value(ind2,:)=[];PF_set.ClID(ind2)=[];PF_set.Type(ind2)=[];
            end
            if ~isequal(unique(ChangedSols.ClID),0) %Added by Shahram Sahraei for cluster-based PA-DDS (Dec. 1, 2017)
                PF_set = ND_check_cluster_prime(PF_set, ChangedSols, num_objs); %Added by Shahram Sahraei for cluster-based PA-DDS (Dec. 1, 2017)
            end
            temp = repmat(obj_flag, size(PF_set.Value,1), 1) .* PF_set.Value(:,1:num_objs);  %Updated by Shahram Sahraei for cluster-based PA-DDS (Dec. 1, 2017)
            temp = double(temp);
            save nd_pts.mat temp -ascii
            temp = PF_set.Value(:,num_objs+1:num_objs+num_dec);  %Updated by Shahram Sahraei for cluster-based PA-DDS (Dec. 1, 2017)
            temp = double(temp);
            save nd_sol.mat temp -ascii
            temp = PF_set.ClID;  %Added by Shahram Sahraei for cluster-based PA-DDS (Dec. 1, 2017)
            temp = double(temp);   %Added by Shahram Sahraei for cluster-based PA-DDS (Dec. 1, 2017)
            save nd_ClID.mat temp -ascii   %Added by Shahram Sahraei for cluster-based PA-DDS (Dec. 1, 2017)
            temp = PF_set.Type;  %Added by Shahram Sahraei for cluster-based PA-DDS (Dec. 1, 2017)
            temp = double(temp);   %Added by Shahram Sahraei for cluster-based PA-DDS (Dec. 1, 2017)
            save nd_Type.mat temp -ascii   %Added by Shahram Sahraei for cluster-based PA-DDS (Dec. 1, 2017)
        end
        if out_plot == 1
           Plot_tradeoff(obj_flag, PF_set.Value, [], [], objfunc_name, i, maxiter);  %Updated Cluster
           pause(0.001);
        end
    end
end
%% Main Loop of PA-DDS
ileft = maxiter - its;
%selection method: 0: Random (RND)
                  %1: Normalized Hyper-volume (for each cluster in normalized Objective Space)+Dissimilarity Index (for each cluster in normalized decision space)
                  %2: Knee-point Distance from Nadir Point (for each cluster in normalized objective space) + Dissimilarity Index (for each cluster in normalized decision space)
switch Select_metric
    case {0}
        Metric = ones(size(PF_set.Value, 1), 1); %Updated Cluster
    case {1}
        Metric = HV_Cluster(PF_set,num_objs,S_min,S_max); %Hypervolume %Updated Cluster
    case {2}
        Metric=DistanceMetric(PF_set,num_objs,S_min,S_max);
end
for i = 1 : ileft  % remaining F evals after initialization
    i
    if mod(i,100) == 0
        fprintf('   iteration %d #Archived_sol %d\n', i, size(PF_set.Value, 1)); %updated cluster
    end
    Index = roulette_wheel(Metric);
    sbest = PF_set.Value(Index, num_objs + 1 : num_objs + num_dec);  %Updated by Shahram Sahraei for cluster-based PA-DDS (Dec. 1, 2017)
    Jbest = PF_set.Value(Index, 1 : num_objs);  %Updated by Shahram Sahraei for cluster-based PA-DDS (Dec. 1, 2017)

%% Plotting 
    if out_plot == 1
        Plot_tradeoff(obj_flag, PF_set.Value, [], Jbest, objfunc_name, its+i, maxiter);  %Updated Cluster
        pause(0.001);
    end
%% DDS
    Pn = 1.0 - log(i) / log(ileft);
    dvn_count = 0; % counter for how many decision variables vary in neighbour
    randnums = rand(num_dec,1);
    stest = sbest;
    for j = 1 : num_dec
        if randnums(j) < Pn % then j th DV selected to vary in neighbour
            dvn_count = dvn_count + 1;
            new_value = neigh_value_mixed(sbest(j), S_min(j), S_max(j), fraction1, j);
            stest(j) = new_value; % change relevant dec var value in stest
        end
    end
    if dvn_count == 0 % no DVs selected at random, so select ONE
        dec_var = ceil(num_dec * rand(1,1)); % which dec var to modify for neighbour   
        new_value = neigh_value_mixed(sbest(dec_var), S_min(dec_var), S_max(dec_var), fraction1, dec_var);
        stest(dec_var) = new_value; % change relevant dec var value in stest
    end
    cd(Modeldir);
    Jtest = feval(objfunc_name,stest);
    Jtest = obj_flag .* Jtest;
    NewSol.Value=[Jtest, stest]; %Added by Shahram Sahraei for cluster-based PA-DDS (Dec. 1, 2017)
    NewSol.ClID=0; %Added by Shahram Sahraei for cluster-based PA-DDS (Dec. 1, 2017)
    NewSol.Type=0; %Added by Shahram Sahraei for cluster-based PA-DDS (Dec. 1, 2017)
    SetOfSols=struct('Value',PF_set.Value(:,num_objs+1:end),'ClID',PF_set.ClID','Type',PF_set.Type'); %Added by Shahram Sahraei for cluster-based PA-DDS (Dec. 1, 2017)
    NewOne=struct('Value',NewSol.Value(num_objs+1:end),'ClID',NewSol.ClID,'Type',NewSol.Type); %Added by Shahram Sahraei for cluster-based PA-DDS (Dec. 1, 2017)
    SetOfSols.Value=(SetOfSols.Value-repmat(S_min,size(SetOfSols.Value,1),1))./repmat(S_max-S_min,size(SetOfSols.Value,1),1); %Added by Shahram Sahraei for cluster-based PA-DDS (Dec. 1, 2017)
    NewOne.Value=(NewOne.Value-S_min)./(S_max-S_min); %Added by Shahram Sahraei for cluster-based PA-DDS (Dec. 1, 2017)
    [Logic, SetOfSols,ChangedSols,Last_label]=ExpandCluster(SetOfSols,NewOne,NewClID,Eps,MinPts); %Added by Shahram Sahraei for cluster-based PA-DDS (Dec. 1, 2017)
%%%%%    [SetOfSols.ClID, Last_label, NewOne.ClID]=relabel1(SetOfSols.ClID);  %Added by Shahram Sahraei for cluster-based PA-DDS (Dec. 1, 2017)
    SetOfSols.Value=(SetOfSols.Value.*repmat(S_max-S_min,size(SetOfSols.Value,1),1)+repmat(S_min,size(SetOfSols.Value,1),1)); %Added by Shahram Sahraei for cluster-based PA-DDS (Dec. 1, 2017)
    ChangedSols.Value=(ChangedSols.Value.*repmat(S_max-S_min,size(ChangedSols.Value,1),1)+repmat(S_min,size(ChangedSols.Value,1),1)); %Added by Shahram Sahraei for cluster-based PA-DDS (Dec. 1, 2017)
    NewSol.ClID=SetOfSols.ClID(end);   %Added by Shahram Sahraei for cluster-based PA-DDS (Dec. 1, 2017)
    NewSol.Type=SetOfSols.Type(end);   %Added by Shahram Sahraei for cluster-based PA-DDS (Dec. 1, 2017)
    PF_set.Value=[PF_set.Value;NewSol.Value]; %Added by Shahram Sahraei for cluster-based PA-DDS (Dec. 1, 2017)
    PF_set.ClID=SetOfSols.ClID'; %Added by Shahram Sahraei for cluster-based PA-DDS (Dec. 1, 2017)
    PF_set.Type=SetOfSols.Type'; %Added by Shahram Sahraei for cluster-based PA-DDS (Dec. 1, 2017)
    ind=ismembertol(PF_set.Value(:,num_objs+1:end),ChangedSols.Value,0.00000001,'ByRows',true); %Added by Shahram Sahraei for cluster-based PA-DDS (Dec. 1, 2017)
    ind2=find(ind==1); %Added by Shahram Sahraei for cluster-based PA-DDS (Dec. 1, 2017)
    ind3=PF_set.ClID(ind2)~=unique(ChangedSols.ClID);
    ind2(ind3)=[];
    a=struct('Value',PF_set.Value(ind2,:),'ClID',PF_set.ClID(ind2),'Type',PF_set.Type(ind2)); %Added by Shahram Sahraei for cluster-based PA-DDS (Dec. 1, 2017)
    [ChangedSols.Value,ia,~]=unique(a.Value,'rows','stable');
    ChangedSols.ClID=a.ClID(ia);
    ChangedSols.Type=a.Type(ia);
    if isequal(Logic,1)  %Added by Shahram Sahraei for cluster-based PA-DDS (Dec. 1, 2017)
        NewClID=Last_label+1;  %Added by Shahram Sahraei for cluster-based PA-DDS (Dec. 1, 2017)
    end   
    cd(main_dir);
    fac=1;
    if size(a.Value,1)>1
        temp1=ChangedSols;
        ChangedSols = ND_check_cluster_prime(ChangedSols, ChangedSols, num_objs); %Added by Shahram Sahraei for cluster-based PA-DDS (Dec. 1, 2017)      
        ind=ismembertol(temp1.Value(:,1:num_objs),ChangedSols.Value(:,1:num_objs),0.00000001,'Byrows',true);
        ind2=find(ind==1);
        temp1.Value(ind2,:)=[];temp1.ClID(ind2)=[];temp1.Type(ind2)=[];
        ind=ismembertol(PF_set.Value(:,1:num_objs),temp1.Value(:,1:num_objs),0.00000001,'Byrows',true);
        ind2=find(ind==1);
        ind3=PF_set.ClID(ind2)~=unique(temp1.ClID);
        ind2(ind3)=[];
        PF_set.Value(ind2,:)=[];PF_set.ClID(ind2)=[];PF_set.Type(ind2)=[];
    elseif ~isequal(a.ClID,0) && ~isempty(a.ClID) %Added by Shahram Sahraei for cluster-based PA-DDS (Dec. 1, 2017)
        Merge=find(PF_set.ClID==ChangedSols.ClID); %Added by Shahram Sahraei for cluster-based PA-DDS (Dec. 1, 2017)
        PF_set_prime=struct('Value',PF_set.Value(Merge,:),'ClID',PF_set.ClID(Merge),'Type',PF_set.Type(Merge)); %Added by Shahram Sahraei for cluster-based PA-DDS (Dec. 1, 2017)
        PF_set.Value(Merge,:)=[]; %Added by Shahram Sahraei for cluster-based PA-DDS (Dec. 1, 2017)
        PF_set.ClID(Merge)=[]; %Added by Shahram Sahraei for cluster-based PA-DDS (Dec. 1, 2017)
        PF_set.Type(Merge)=[]; %Added by Shahram Sahraei for cluster-based PA-DDS (Dec. 1, 2017)
        PF_set_prime = ND_check_cluster_prime(PF_set_prime, ChangedSols, num_objs); %Added by Shahram Sahraei for cluster-based PA-DDS (Dec. 1, 2017)
        PF_set.Value=[PF_set.Value;PF_set_prime.Value]; %Added by Shahram Sahraei for cluster-based PA-DDS (Dec. 1, 2017)
        PF_set.ClID=[PF_set.ClID PF_set_prime.ClID]; %Added by Shahram Sahraei for cluster-based PA-DDS (Dec. 1, 2017)
        PF_set.Type=[PF_set.Type PF_set_prime.Type]; %Added by Shahram Sahraei for cluster-based PA-DDS (Dec. 1, 2017)
        fac=0;
    end
    if fac==1
        PF_set = ND_check_cluster_prime(PF_set, ChangedSols, num_objs); %Added by Shahram Sahraei for cluster-based PA-DDS (Dec. 1, 2017)
    end
    temp = repmat(obj_flag, size(PF_set.Value,1), 1) .* PF_set.Value(:,1:num_objs); %Added by Shahram Sahraei for cluster-based PA-DDS (Dec. 1, 2017)
    temp = double(temp);
    save nd_pts.mat temp -ascii
    temp = PF_set.Value(:, num_objs + 1 : num_objs + num_dec); 
    temp = double(temp);
    save nd_sol.mat temp -ascii;
    temp = PF_set.ClID;  %Added by Shahram Sahraei for cluster-based PA-DDS (Dec. 1, 2017)
    temp = double(temp);  %Added by Shahram Sahraei for cluster-based PA-DDS (Dec. 1, 2017)
    save nd_ClID.mat temp -ascii;  %Added by Shahram Sahraei for cluster-based PA-DDS (Dec. 1, 2017)
    temp = PF_set.Type;  %Added by Shahram Sahraei for cluster-based PA-DDS (Dec. 1, 2017)
    temp = double(temp);  %Added by Shahram Sahraei for cluster-based PA-DDS (Dec. 1, 2017)
    save nd_Type.mat temp -ascii;  %Added by Shahram Sahraei for cluster-based PA-DDS (Dec. 1, 2017)
    switch Select_metric
        case {0}
            Metric = ones(size(PF_set.Value, 1), 1); % Random Selection  %Updated Cluster
        case {1}
            Metric = HV_Cluster(PF_set,num_objs,S_min,S_max); %Hypervolume %Updated Cluster
        case {2}
            Metric=DistanceMetric(PF_set,num_objs,S_min,S_max);
    end
    if out_plot == 1
        Plot_tradeoff(obj_flag, PF_set.Value, Jtest, Jbest, objfunc_name, its+i, maxiter);  %Updated Cluster
        pause(0.001);
    end
end % DDS function loop


