function Metric=DistanceMetric(Points,num_objs,S_min,S_max)
% This is a new method for the calculation of tradeoffs' hypervolume by
% taking decision-space diversity into consideration. This function adjusts
% the value of tradeoff's hypervolume in the objective space based on the
% level of dissimilarity that the cluster has with other clusters in the
% decision space. This function requires a dissimilarity level value to be
% difined by the user ahead of calculation. The Eucledian distance between
% cluster centers are calculated and if the Eucledian distance between two
% cluster centers are less than the dissimilarity level, there will be a
% link between two clusters and they are considered as relatively similar
% clusters. The lower the number of connections the greater the adjustment
% value and the less adjustment is needed for the hypervolume value of the
% cluster.
Metric=zeros(length(Points.ClID),1);
labels=unique(Points.ClID,'sorted');
first_label=labels(1);
last_label=labels(end);
if first_label==0
    loc=find(Points.ClID==0);
    NumOfZeroIDs=length(loc);
else 
    NumOfZeroIDs=0;
end
%% Knee Point Calculation
NadirPoint = -inf * ones(1, num_objs);
UtopiaPoint = inf * ones(1, num_objs);
MAX = max(Points.Value(:,1:num_objs), [], 1);
Index = find(MAX > NadirPoint);
NadirPoint(Index) = MAX(Index);
MIN = min(Points.Value(:,1:num_objs), [], 1);
Index = find(MIN < UtopiaPoint);
UtopiaPoint(Index) = MIN(Index);
NrmlzdObjPts=(Points.Value(:,1:num_objs)-repmat(UtopiaPoint,size(Points.Value,1),1))./repmat((NadirPoint-UtopiaPoint),size(Points.Value,1),1);
Knee=zeros(last_label+NumOfZeroIDs,size(Points.Value,2));
KneeDist=zeros(last_label+NumOfZeroIDs,1);
for i=first_label:last_label
    if i==0
        KneeDist(1:NumOfZeroIDs)=pdist2(NrmlzdObjPts(loc,:),zeros(1,length(UtopiaPoint)),'euclidean');
        Knee(1:NumOfZeroIDs,1:num_objs)=NrmlzdObjPts(loc,:);
        Knee(1:NumOfZeroIDs,num_objs+1:end)=Points.Value(loc,num_objs+1:end);
    else
        loc=find(Points.ClID==i);
        a=NrmlzdObjPts(loc,:);
        distance=pdist2(a,zeros(1,length(UtopiaPoint)),'euclidean');
        KneeDist(i+NumOfZeroIDs)=min(distance);
        loc2=find(distance==min(distance));
        Knee(i+NumOfZeroIDs,1:num_objs)=NrmlzdObjPts(loc(loc2(1)),:);
        Knee(i+NumOfZeroIDs,num_objs+1:end)=Points.Value(loc(loc2(1)),num_objs+1:end);
    end
end
%%
KneeDistPrime=1-KneeDist/sqrt(num_objs);
Knee(:,num_objs+1:end)=(Knee(:, num_objs+1:end)-repmat(S_min,size(Knee,1),1))./(repmat((S_max-S_min),size(Knee,1),1));
edge=pdist(Knee(:,num_objs+1:end),'euclidean')/sqrt(length(S_min));
mat_edge=tril(ones(NumOfZeroIDs+last_label),-1);
mat_edge(mat_edge~=0)=edge;
mat_edge=mat_edge+tril(mat_edge,-1).';
farness=(sum(mat_edge))';
Dissim=(farness-repmat(min(farness),length(farness),1))./repmat(max(farness)-min(farness),length(farness),1);
Met=Dissim+KneeDistPrime;
for i=first_label:last_label
    loc=find(Points.ClID==i);
    if i==0
        Metric(loc)=Met(1:NumOfZeroIDs);
    else
        Metric(loc)=Met(NumOfZeroIDs+i);
    end
end