function HV_Clus=HV_Cluster(Points,num_objs,S_min,S_max)
% This is a new method for the calculation of tradeoffs' hypervolume by
% taking decision-space diversity into consideration. This function adjusts
% the value of tradeoff's hypervolume in the objective space based on the
% level of dissimilarity that the cluster has with other clusters in the
% decision space. This function requires a dissimilarity level value to be
% difined by the user ahead of calculation. The Eucledian distance between
% cluster centers are calculated and if the Eucledian distance between two
% cluster centers are less than the dissimilarity level, there will be a
% link between two clusters and they are considered as relatively similar
% clusters. The lower the number of connections the greater the adjustment
% value and the less adjustment is needed for the hypervolume value of the
% cluster.
HV_Clus=zeros(length(Points.ClID),1);
labels=unique(Points.ClID,'sorted');
first_label=labels(1);
last_label=labels(end);
if first_label==0
    loc=find(Points.ClID==0);
    NumOfZeroIDs=length(loc);
else 
    NumOfZeroIDs=0;
end
HV_NEW = ones(last_label+NumOfZeroIDs, 1);
for i=first_label:last_label
    if i==0
        for j=1:NumOfZeroIDs
            HV_NEW(j) = Norm_HV(Points.Value(loc(j), 1: num_objs),Points.Value(:, 1: num_objs));
        end
    else
        loc=find(Points.ClID==i);
        HV_NEW(i+NumOfZeroIDs) = Norm_HV(Points.Value(loc, 1: num_objs),Points.Value(:, 1: num_objs));
    end
end
NDV=(Points.Value(:, num_objs+1:end)-repmat(S_min,size(Points.Value,1),1))./(repmat((S_max-S_min),size(Points.Value,1),1));
ClCenter=zeros(NumOfZeroIDs+last_label,size(NDV,2));
for i=first_label:last_label
    loc=find(Points.ClID==i);
    if i~=0
        ClCenter(NumOfZeroIDs+i,:)=mean(NDV(loc, :));
    else
        for j=1:NumOfZeroIDs
            ClCenter(j,:)=NDV(loc(j), :);
        end
    end
end
if size(ClCenter,1)>1 % There are at least one cluster and one noise in decision variable space.
    edge=pdist(ClCenter,'euclidean')/sqrt(size(NDV,2));
    mat_edge=tril(ones(NumOfZeroIDs+last_label),-1);
    mat_edge(mat_edge~=0)=edge;
    mat_edge=mat_edge+tril(mat_edge,-1).';
    farness=(sum(mat_edge))';
    if length(unique(farness))~=1
        Dissim=(farness-repmat(min(farness),length(farness),1))./repmat(max(farness)-min(farness),length(farness),1);
    else
        Dissim=1;
    end
else % The cluster spatial density domain is too big and only one cluster is formed.
    Dissim=0;
end
HV_Dissim=Dissim+HV_NEW/max(HV_NEW);
for i=first_label:last_label
    loc=find(Points.ClID==i);
    if i==0
        HV_Clus(loc)=HV_Dissim(1:NumOfZeroIDs);
    else
        HV_Clus(loc)=HV_Dissim(NumOfZeroIDs+i);
    end
end