function ND_set = ND_check_cluster_prime(ND_set, ChangedSols,num_objs)
% Written by Shahram Sahraei 2017-2018
% This function gets a set of cluster-non-dominated solutions in "ND_set", 
% "ChangedSols" containing solutions with updated cluster labels after
% applying the ExpandCluster function to the archived solutions as well as
% new solution, and "num_objs" denoting the number of objective functions.
% Solutions with the same cluster labels are found in ND_set and they are
% compared in terms of dominance relations. Solutions with distinct cluster
% labels are not checked in terms of domination.
% Assume X1 and X2 have the same cluster labels:
%   if dominance_flag = -1: X1 is cluster-dominated by X2
%   if dominance_flag =  0: X1 and X2 are both cluster-non-dominated
%   if dominanceu_flag =  1: X1 is cluster-dominating and X2 is
%                         cluster-dominated.
% ND_set is updated by removing the cluster-dominated solutions in ND_set.

label=unique(ChangedSols.ClID);
if length(label)~=1
    error('All the recently labelled solutions should have the same ID');
end
if label==0
    if ~ismember(ND_set.Value,ChangedSols.Value,'rows')
        ND_set.Value(end+1,:)=ChangedSols.Value;
        ND_set.ClID(end+1)=ChangedSols.ClID;
        ND_set.Type(end+1)=ChangedSols.Type;
        clear dominance_flag;
%         dominance_flag(1,1,1) = 0;
%     else
%         dominance_flag(1,1,1) = -1;
    end
else
    z=find(ND_set.ClID==label);
    PF_Cluster=ND_set.Value(z,1:num_objs);
%     PS_Cluster=ND_set.Value(z,num_objs+1:end);
    dominance_flag = zeros(size(z,2));

% Either this for loop or the next one.
%     record=[];
%     for i=1:length(z)-1
%         for k=i+1:length(z)
%             num_eql = sum(PF_Cluster(i, 1 : num_objs)== PF_Cluster(k, 1 : num_objs));
%             num_imp = sum(PF_Cluster(i, 1 : num_objs) < PF_Cluster(k, 1 : num_objs));
%             num_deg = sum(PF_Cluster(i, 1 : num_objs) > PF_Cluster(k, 1 : num_objs));
%             if (num_imp == 0) && (num_deg > 0) % X is dominated
%                 dominance_flag(k,i) = -1; % X is dominated
%                      
%             elseif num_eql == num_objs % Objective functions are the same for X and archived solution i
%                 dominance_flag(k,i) = 0;  % X is non-dominated
%                 record=[record;[k,i]];
%             elseif num_imp > 0 && num_deg == 0 % X dominates ith solution in the ND_set
%                 dominance_flag(k,i) = 1;
%             elseif num_imp>0 && num_deg>0 % Objective functions are the same for X and archived solution i
%                 dominance_flag(k,i) = 0;  % X is non-dominated
%             end
%         end
%     end
%     %Find the dominated solutions and delete them from ND_set
%     ff=tril(dominance_flag,-1);
% %     dominance_flag=ff-ff';
%     [r,~]=find(ff==1);
%     [~,c]=find(ff==-1);
%     s=[];
%     if ~isempty(record)
%         s=record(:,1);
%     end
%     f=unique([r;c;s]);
    
    for i=1:length(z)-1
        for k=i+1:length(z)
            num_eql = sum(PF_Cluster(i, 1 : num_objs)== PF_Cluster(k, 1 : num_objs));
            num_imp = sum(PF_Cluster(i, 1 : num_objs) < PF_Cluster(k, 1 : num_objs));
            num_deg = sum(PF_Cluster(i, 1 : num_objs) > PF_Cluster(k, 1 : num_objs));
            if (num_imp == 0) && (num_deg > 0) || (num_eql == num_objs) % X is dominated
                dominance_flag(k,i) = -1; % Solution No. "i" is dominated
                break;
            elseif num_imp > 0 && num_deg == 0 % Solution No. "i" is dominating
                dominance_flag(k,i) = 1;
            end
        end
    end
    %Find the dominated solutions and delete them from ND_set
    ff=tril(dominance_flag,-1);
    [r,~]=find(ff==1);
    [~,c]=find(ff==-1);
    f=unique([r;c]);
    if ~isempty(f)
        ND_set.Value(z(f),:)=[];
        ND_set.ClID(z(f))=[];
        ND_set.Type(z(f))=[];
    end

end