function [Seeds,SetOfSols,NewSol]=RegionQuery(SetOfSols,NewSol,Eps)
% [Seeds,SetOfSols,NewSol]=RegionQuery(SetOfSols,NewSol,Eps)
%
%
% This function seeks all the solutions in the range of Eps neighborhood of
% the new point, here known as NewSol. In other words, a hypersphere, with
% radius of Eps and NewSol in the center, is drawn and all the cluster
% labels and points whithin hypersphere are stored in Seeds.
%
% Inputs:
%       SetOfSols: Data/solutions information structure (created so far)
%
%               SetOfSols.Value: Data/Solution matrix (rows solutions,
%               columns decision variables)
%
%               SetOfSols.ClID: Solutions' cluster labels
%
%               SetOfSols.Type: Cumulative adjacency density from the
%               beginning of optimization for the archived solutions
%
%       NewSol: The new generated solution with the same structure and
%               fields as SetOfSols
%
%
%       Eps: range of the neighborhood
%
%
% Output:
%
%       Seeds:  Nearby solutions of structure type that are reachable
%                from the new solution "NewSol"
%
%               Seeds.Value: Nearby solutions matrix (rows solutions,
%               columns decision variables)
%
%               Seeds.ClID: Cluster labels of the nearby solutions
%
%               Seeds.Type: Cumulative adjacency density of nearby
%               solutions
%
%               SetOfSols: the updated SetOfSols after introducing NewSol
%               to the variable space
%
%               NewSol: The updated NewSol mainly in terms of cumulative
%               adjacency density
%

%%
a=(SetOfSols.Value-repmat(NewSol.Value,size(SetOfSols.Value,1),1)).^2;
a=sum(a,2);
dist=sqrt(a);
lab=find(dist<=Eps);
if NewSol.Type==0
    % Archive new solution in the Seeds since it covers itself.
    SetOfSols.Type(lab)=SetOfSols.Type(lab)+1;
    Seeds.Value=[NewSol.Value;SetOfSols.Value(lab,:)];
    Seeds.ClID=[NewSol.ClID;SetOfSols.ClID(lab)];
    Seeds.Type=[NewSol.Type;SetOfSols.Type(lab)];
    NewSol.Type=size(Seeds.Value,1);
    Seeds.Type(1)=NewSol.Type;
else
    Seeds.Value=SetOfSols.Value(lab,:);
    Seeds.ClID=SetOfSols.ClID(lab);
    Seeds.Type=SetOfSols.Type(lab);
end