function snew = neigh_value_continuous(s,s_min,s_max,fraction1)

% select a RANDOM neighbouring real value of a SINGLE decision variable
% CEE 509, HW 5 by Bryan Tolson, Mar 5, 2003 AND ALSO CEE PROJECT

% variables:
% s is a current SINGLE decision variable VALUE
% s_min is the min of variable s
% s_max is the max of variable s
% snew is the neighboring VALUE of the decision variable
% fraction1 is the neighbourhood parameter (replaces V parameter~see notes)
%           It is defined as the ratio of the std deviation of the desired 
%           normal random number/s_range.  Eg:
%               std dev desired = fraction1 * s_range
%               for comparison:  variance (V) = (fraction1 * s_range)^2
% s_range is the range of the real variable (s_max-s_min)

s_range = s_max-s_min;

snew = s + randn(1,1) * fraction1 * s_range;

% NEED to deal with variable upper and lower bounds:
% Originally bounds in DDS were 100% reflective
% But some times DVs are right on the boundary and with 100% reflective
% boundaries it is hard to detect them. Therefore, we decided to make the
% boundaries reflective with 50% chance and absorptive with 50% chance.
% M. Asadzadeh and B. Tolson Dec 2008
P_Abs_or_Ref = rand();
if snew < s_min % works for any pos or neg s_min
    if P_Abs_or_Ref <= 0.5%with 50% chance reflect
        snew = s_min + (s_min - snew); 
    else % with 50% chance absorb
        snew = s_min;
    end
        % if reflection goes past s_max then value should be s_min since without reflection
        % the approach goes way past lower bound.  This keeps X close to lower bound when X current
        % is close to lower bound:
    if snew > s_max
        snew = s_min;
    end
elseif snew > s_max % works for any pos or neg s_max
    if P_Abs_or_Ref <= 0.5%with 50% chance reflect
        snew = s_max - (snew - s_max); 
    else % with 50% chance absorb
        snew = s_max;
    end
        % if reflection goes past s_min then value should be s_max for same reasons as above
    if snew < s_min
        snew = s_max;
    end
end

% see 'neighbourhood bounding.xls' file for more analysis on above approach.

% Note the probability of sampling exactly at an endpoint based on St. Normal RV is:
%   = range because s current is fixed
%   = [Pr if at boundary of s Pr if at median of s]
%   = [Pmin(snew<min) + Pmin(snew>max), Pmax(snew<min) + Pmax(snew>max)]
%   = [    2*P(Z>2*s_range)           ,  2*P(Z>1.5*s_range)            ] %
%           Note: fraction1=stdev/s_range = 1/s_range, thus s_range = 1/fraction1
%   = [ 2*P(Z>2/fraction1), 2*P(Z>1.5/fraction1)]
% Table of resulting probabilities:
%   fraction1   Pmin        Pmax
%   <0.3        <0.0000     <0.0000 
%   0.4000      0.0000      0.0002
%   0.6000      0.0009      0.0124
%   0.8000      0.0124      0.0608
%   1.0000      0.0455      0.1336
%   1.2000      0.0956      0.2113

% above makes it difficult for neighbours to reach their absolute max and min 
% when fraction1 <0.3 but they can get close.  based on above, a
% fraction>1.2 does not seem to make very much sense since 1 of every 5 to 10
% samples of a parameter will result in visiting an exact endpoint.

